@@ -4,70 +4,19 @@ Provides substitution in all dictionaries throughout the interpreter,
 this includes references held by objects and modules.  Restores those 
 references which were replaced (only), i.e. you can re-use the objects
 you used as substitutions after restoring.
-
-Sample usage:
-
-    >>> import globalsub
-    >>> def x():
-    ...     print 'x'
-    ... 
-    >>> def y():
-    ...     print 'y'
-    ... 
-    >>> globalsub.subs( x, y )
-    <function y at 0xb7470a04>
-    >>> x()
-    y
-    >>> y()
-    y
-    >>> globalsub.restore( x )
-    <function x at 0xb74709cc>
-    >>> x()
-    x
-    >>> y()
-    y
-
-Sample usage for a test case:
-
-    import globalsub as gs
-    class Test(TestCase):
-        def setUp( self ):
-            gs.subs( somefunction, replacement )
-        def tearDown( self ):
-            gs.restore( somefunction )
-
-Notes:
-
-    * if you need to refer to the original function, store a
-      reference in a list or tuple
-    * corollary of that is that you likely do *not* want to store 
-      function references in lists and iterate over them to substitute,
-      if you do, you will need to capture the instances returned from 
-      subs() and do a restore on them.
-    * certain fixed-attribute values (SUBS_TYPES) will be replaced with 
-      copies of the object which can receive arbitrary attributes.  
-      List objects, however, cannot be so wrapped because they are mutable 
-      and you may need their identities to remain the same.  You can 
-      explicitly wrap them with globalsub.List( [] ) to make them 
-      subs-compatible.
-
-    * do *not* use the same object as the value-to-substitute for 
-      more than one name/variable; only the *last* item will be 
-      substituted; you will only be able to restore the last item 
-      subs()'d.
 """
-__version__ = '1.0.1'
+__version__ = '1.0.2'
 import gc,logging 
 log = logging.getLogger(__name__)
 
 __all__ = ['subs','restore','List']
 
 class List( list ):
-    """Use to wrap your lists so that they can be subs'd"""
+    """Use to wrap your lists so that they can be subs'd (deprecated, just pass in lists)"""
 
 SUBS_TYPES = (str,unicode,int,long,float,tuple,complex,set,bool,)
 def subsable( x ):
-    """For builtin types that do not allow arbitrary attributes, subclass and copy"""
+    """For built-in types that do not allow arbitrary attributes, subclass and copy"""
     if x.__class__ in SUBS_TYPES:
         # Note the explicit check for *precisely* these classes, *not* isinstance
         cls = type( 'Subs', (x.__class__,), {} )
@@ -81,7 +30,7 @@ def subs( function, replacement=None ):
     
     function -- function to replace
     replacement -- if provided, is used as the replacement object,
-        otherwise a function returning None is constructed 
+    otherwise a function returning None is constructed 
     
     Used to stub out functionality globally, this function uses 
     global_replace to find all references to function and replace 
@@ -103,12 +52,15 @@ def subs( function, replacement=None ):
     else:
         replacement = subsable( replacement )
     if function is not replacement:
-        replacement.__is_subs__ = True
-        replacement.original = [function,replace_filter(replacement)]
+        _RESTORE_MAPPING[ id(replacement) ] = [function,replace_filter(replacement)]
         global_replace( function, replacement )
         return replacement 
     return function
 
+_RESTORE_MAPPING = {
+    # id(replacement): [ function, replace_filter( replacement ) ],
+}
+
 def restore( function ):
     """Restore previously subs'd function in all namespaces
     
@@ -127,9 +79,9 @@ def resolve( function ):
     _mocker_replace_ = False
     seen = {}
     all_filters = {}
-    while hasattr( function, 'original' ):
+    while id(function) in _RESTORE_MAPPING:
         seen[id(function)] = True 
-        function,filters = function.original
+        function,filters = _RESTORE_MAPPING.pop( id(function))
         all_filters.update( filters )
         if id(function) in seen:
             log.warn( 'Seem to have created a substituation loop on %s', function)
